=== modified file 'NEWS'
--- NEWS	2006-10-04 18:40:24 +0000
+++ NEWS	2006-10-09 09:17:29 +0000
@@ -9,6 +9,10 @@
       can take a ``working_dir='foo'`` parameter, which will change directory 
       for the command. (John Arbash Meinel)
 
+    * ``bzrlib.lazy_regex.lazy_compile`` can be used to create a proxy
+      around a regex, which defers compilation until first use. 
+      (John Arbash Meinel)
+
   BUG FIXES:
 
     * bzr branch/pull no longer complain about progress bar cleanup when

=== modified file 'bzr'
--- bzr	2006-09-29 17:43:15 +0000
+++ bzr	2006-10-09 14:06:24 +0000
@@ -50,7 +50,6 @@
     os.unsetenv(REINVOKE)
 
 
-
 profiling = False
 if '--profile-imports' in sys.argv:
     sys.argv.remove('--profile-imports')
@@ -71,6 +70,9 @@
 import bzrlib.inspect_for_copy
 bzrlib.inspect_for_copy.import_copy_with_hacked_inspect()
 
+import bzrlib.lazy_regex
+bzrlib.lazy_regex.install_lazy_compile()
+
 import bzrlib.commands
 import bzrlib.trace
 

=== added file 'bzrlib/lazy_regex.py'
--- bzrlib/lazy_regex.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lazy_regex.py	2006-10-10 06:48:45 +0000
@@ -0,0 +1,106 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Lazily compiled regex objects.
+
+This module defines a class which creates proxy objects for regex compilation.
+This allows overriding re.compile() to return lazily compiled objects.
+"""
+
+import re
+
+
+class LazyRegex(object):
+    """A proxy around a real regex, which won't be compiled until accessed."""
+
+
+    # These are the parameters on a real _sre.SRE_Pattern object, which we
+    # will map to local members so that we don't have the proxy overhead.
+    _regex_attributes_to_copy = [
+                 '__copy__', '__deepcopy__', 'findall', 'finditer', 'match',
+                 'scanner', 'search', 'split', 'sub', 'subn'
+                 ]
+
+    # We use slots to keep the overhead low. But we need a slot entry for
+    # all of the attributes we will copy
+    __slots__ = ['_real_regex', '_regex_args', '_regex_kwargs',
+                ] + _regex_attributes_to_copy
+
+    def __init__(self, args=(), kwargs={}):
+        """Create a new proxy object, passing in the args to pass to re.compile
+
+        :param args: The *args to pass to re.compile
+        :param kwargs: The **kwargs to pass to re.compile
+        """
+        self._real_regex = None
+        self._regex_args = args
+        self._regex_kwargs = kwargs
+
+    def _compile_and_collapse(self):
+        """Actually compile the requested regex"""
+        self._real_regex = self._real_re_compile(*self._regex_args,
+                                                 **self._regex_kwargs)
+        for attr in self._regex_attributes_to_copy:
+            setattr(self, attr, getattr(self._real_regex, attr))
+
+    def _real_re_compile(self, *args, **kwargs):
+        """Thunk over to the original re.compile"""
+        return _real_re_compile(*args, **kwargs)
+
+    def __getattr__(self, attr):
+        """Return a member from the proxied regex object.
+
+        If the regex hasn't been compiled yet, compile it
+        """
+        if self._real_regex is None:
+            self._compile_and_collapse()
+        # Once we have compiled, the only time we should come here
+        # is actually if the attribute is missing.
+        return getattr(self._real_regex, attr)
+
+
+def lazy_compile(*args, **kwargs):
+    """Create a proxy object which will compile the regex on demand.
+
+    :return: a LazyRegex proxy object.
+    """
+    return LazyRegex(args, kwargs)
+
+
+def install_lazy_compile():
+    """Make lazy_compile the default compile mode for regex compilation.
+
+    This overrides re.compile with lazy_compile. To restore the original
+    functionality, call reset_compile().
+    """
+    re.compile = lazy_compile
+
+
+def reset_compile():
+    """Restore the original function to re.compile().
+    
+    It is safe to call reset_compile() multiple times, it will always
+    restore re.compile() to the value that existed at import time.
+    Though the first call will reset back to the original (it doesn't
+    track nesting level)
+    """
+    re.compile = _real_re_compile
+
+
+_real_re_compile = re.compile
+assert _real_re_compile is not lazy_compile, \
+    "re.compile has already been overridden as lazy_compile, but this would" \
+    " cause infinite recursion"

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-10-04 03:25:43 +0000
+++ bzrlib/tests/__init__.py	2006-10-09 09:17:29 +0000
@@ -1618,6 +1618,7 @@
                    'bzrlib.tests.test_inv',
                    'bzrlib.tests.test_knit',
                    'bzrlib.tests.test_lazy_import',
+                   'bzrlib.tests.test_lazy_regex',
                    'bzrlib.tests.test_lockdir',
                    'bzrlib.tests.test_lockable_files',
                    'bzrlib.tests.test_log',

=== added file 'bzrlib/tests/test_lazy_regex.py'
--- bzrlib/tests/test_lazy_regex.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_lazy_regex.py	2006-10-09 09:41:01 +0000
@@ -0,0 +1,129 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that lazy regexes are not compiled right away"""
+
+import re
+
+from bzrlib import (
+    lazy_regex,
+    tests,
+    )
+
+
+class InstrumentedLazyRegex(lazy_regex.LazyRegex):
+    """Keep track of actions on the lazy regex"""
+
+    _actions = []
+
+    @classmethod
+    def use_actions(cls, actions):
+        cls._actions = actions
+
+    def __getattr__(self, attr):
+        self._actions.append(('__getattr__', attr))
+        return super(InstrumentedLazyRegex, self).__getattr__(attr)
+
+    def _real_re_compile(self, *args, **kwargs):
+        self._actions.append(('_real_re_compile',
+                                               args, kwargs))
+        return super(InstrumentedLazyRegex, self)._real_re_compile(*args, **kwargs)
+
+
+class TestLazyRegex(tests.TestCase):
+
+    def test_lazy_compile(self):
+        """Make sure that LazyRegex objects compile at the right time"""
+        actions = []
+        InstrumentedLazyRegex.use_actions(actions)
+
+        pattern = InstrumentedLazyRegex(args=('foo',))
+        actions.append(('created regex', 'foo'))
+        # This match call should compile the regex and go through __getattr__
+        pattern.match('foo')
+        # But a further call should not go through __getattr__ because it has
+        # been bound locally.
+        pattern.match('foo')
+
+        self.assertEqual([('created regex', 'foo'),
+                          ('__getattr__', 'match'),
+                          ('_real_re_compile', ('foo',), {}),
+                         ], actions)
+
+
+class TestLazyCompile(tests.TestCase):
+
+    def test_simple_acts_like_regex(self):
+        """Test that the returned object has basic regex like functionality"""
+        pattern = lazy_regex.lazy_compile('foo')
+        self.assertIsInstance(pattern, lazy_regex.LazyRegex)
+        self.assertTrue(pattern.match('foo'))
+        self.assertIs(None, pattern.match('bar'))
+
+    def test_extra_args(self):
+        """Test that extra arguments are also properly passed"""
+        pattern = lazy_regex.lazy_compile('foo', re.I)
+        self.assertIsInstance(pattern, lazy_regex.LazyRegex)
+        self.assertTrue(pattern.match('foo'))
+        self.assertTrue(pattern.match('Foo'))
+
+    def test_findall(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        self.assertEqual(['f', 'fo', 'foo', 'fooo'],
+                         pattern.findall('f fo foo fooo'))
+
+    def test_finditer(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        matches = [(m.start(), m.end(), m.group())
+                   for m in pattern.finditer('foo bar fop')]
+        self.assertEqual([(0, 3, 'foo'), (8, 10, 'fo')], matches)
+
+    def test_match(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        self.assertIs(None, pattern.match('baz foo'))
+        self.assertEqual('fooo', pattern.match('fooo').group())
+
+    def test_search(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        self.assertEqual('foo', pattern.search('baz foo').group())
+        self.assertEqual('fooo', pattern.search('fooo').group())
+
+    def test_split(self):
+        pattern = lazy_regex.lazy_compile('[,;]*')
+        self.assertEqual(['x', 'y', 'z'], pattern.split('x,y;z'))
+
+
+class TestInstallLazyCompile(tests.TestCase):
+
+    def setUp(self):
+        super(TestInstallLazyCompile, self).setUp()
+        self.addCleanup(lazy_regex.reset_compile)
+
+    def test_install(self):
+        lazy_regex.install_lazy_compile()
+        pattern = re.compile('foo')
+        self.assertIsInstance(pattern, lazy_regex.LazyRegex)
+
+    def test_reset(self):
+        lazy_regex.install_lazy_compile()
+        lazy_regex.reset_compile()
+        pattern = re.compile('foo')
+        self.failIf(isinstance(pattern, lazy_regex.LazyRegex),
+                    'lazy_regex.reset_compile() did not restore the original'
+                    ' compile() function %s' % (type(pattern),))
+        # but the returned object should still support regex operations
+        m = pattern.match('foo')
+        self.assertEqual('foo', m.group())

=== modified file 'profile_imports.py'
--- profile_imports.py	2006-09-12 22:02:02 +0000
+++ profile_imports.py	2006-10-09 14:01:53 +0000
@@ -117,17 +117,17 @@
     extra = ''
     if frame_name.endswith('demandload'):
         # If this was demandloaded, we have 3 frames to ignore
-        extra = ' (demandload)'
+        extra = '(demandload) '
         frame = sys._getframe(4)
         frame_name = frame.f_globals.get('__name__', '<unknown>')
     elif frame_name.endswith('lazy_import'):
         # If this was lazily imported, we have 3 frames to ignore
-        extra = ' (lazy)'
+        extra = '[l] '
         frame = sys._getframe(4)
         frame_name = frame.f_globals.get('__name__', '<unknown>')
     frame_lineno = frame.f_lineno
 
-    this = stack_add(name+extra, frame_name, frame_lineno, scope_name)
+    this = stack_add(extra + name, frame_name, frame_lineno, scope_name)
 
     tstart = time.time()
     try:
@@ -148,9 +148,15 @@
     # And who is requesting this?
     frame = sys._getframe(2)
     frame_name = frame.f_globals.get('__name__', '<unknown>')
+
+    extra = ''
+    if frame_name.endswith('lazy_regex'):
+        # If this was lazily compiled, we have 3 more frames to ignore
+        extra = '[l] '
+        frame = sys._getframe(5)
+        frame_name = frame.f_globals.get('__name__', '<unknown>')
     frame_lineno = frame.f_lineno
-
-    this = stack_add(repr(args[0]), frame_name, frame_lineno)
+    this = stack_add(extra+repr(args[0]), frame_name, frame_lineno)
 
     tstart = time.time()
     try:

